#' Print cluscor output in a pretty format
#'
#' @param x Object of class \code{cluscor} resulting from \code{cluscor.test}
#' @param digits Number of digits in output.
#' @param prefix String prefix to be used prior to printing of method.
#' @param ... Additional arguments to \code{print.default}
#'
#' @return Results of \code{cluscor.test}.
#' @keywords internal
#' @export
#'
#' @examples
#' NULL
print.cluscor <- function (x, digits = getOption("digits"), prefix = "\t", ...){
  cat("\n")
  cat(strwrap(x$method, prefix = prefix), sep = "\n")
  cat("\n")
  if(!is.null(x$fixed)){
    if(x$fixed) {
      cat("Denominator of the variance estimate treated as fixed.\n", sep = "")
    } else {
      cat("Denominator of the variance estimate treated as random.\n", sep = "")
    }
  }
  out <- character()
  cat("number of observations: ", x$nobs, ";  ", sep = "")
  cat("number of clusters: ", x$n.clust, "\n", sep = "")
  if (!is.null(x$statistic))
    out <- c(out, paste(names(x$statistic), "=", format(signif(x$statistic,
                                                               max(1L, digits - 2L)))))
  if (!is.null(x$parameter))
    out <- c(out, paste(names(x$parameter), "=", format(signif(x$parameter,
                                                               max(1L, digits - 2L)))))
  if (!is.null(x$p.value)) {
    fp <- format.pval(x$p.value, digits = max(1L, digits - 3L))
    out <- c(out, paste("p-value",
                        if (substr(fp, 1L, 1L) == "<") fp else paste("=", fp),
                        if (!is.null(x$dist)) paste0(" (estimated with ", x$dist, " distribution)\n")))
    # if(!is.null(x$dist)){
    #   out <- c(out, cat(" (estimated with ", x$dist, " distribution)\n", sep = ""))
    # }
  }
  cat(strwrap(paste(out, collapse = ", ")), sep = "\n")
  if (!is.null(x$conf.int)) {
    if(x$dist == "normal"){
      cat(format(100 * (1-attr(x$conf.int, "conf.level"))), " percent confidence interval:\n",
          " ", paste(format(c(x$conf.int[1L], x$conf.int[2L])),
                     collapse = " "), "\n", sep = "")
    }
    if(x$dist == "t"){
      cat(format(100 * (1-attr(x$conf.int, "conf.level"))), " percent confidence interval (df = ",attr(x$conf.int, "df"),"):\n",
          " ", paste(format(c(x$conf.int[1L], x$conf.int[2L])),
                     collapse = " "), "\n", sep = "")
    }
  }
  if (!is.null(x$estimate)) {
    cat("sample estimates:\n")
    print(x$estimate, digits = digits, ...)
  }
  cat("\n")
  invisible(x)
}
