#' Utility Functions
#'
#' Internal functions for use by cluscor main functions.
#'
#' @keywords internal
#' @name cluscor_utilities
NULL

#' @rdname cluscor_utilities
aggrMean <- function(mat, by) {
  uby <- unique(by)
  p <- ncol(mat)
  nuby <- length(uby)
  out <- matrix(NA, nuby, p)
  for (i in 1:nuby) {
    nl <- sum(by == uby[i])
    out[i, ] <- colMeans(matrix(mat[by == uby[i], ], nr = nl))
  }
  out
}


#' @rdname cluscor_utilities
ci <- function(est, se, alpha, dist = c("normal","t"), df = NULL) {
  dist <- match.arg(dist)
  if(dist == "t" & is.null(df)){stop("CI calculation: for t-distribution, df must be supplied.")}
  upp <- low <- rep(NA, length(est) * length(se))
  k = 1
  for (i in 1:length(est)) {
    for (j in 1:length(se)) {
      if(dist == "normal"){
        upp[k] <- est[i] + qnorm(1 - alpha/2) * se[j]
        low[k] <- est[i] - qnorm(1 - alpha/2) * se[j]
      }
      if(dist == "t"){
        upp[k] <- est[i] + qt(1 - alpha/2, df) * se[j]
        low[k] <- est[i] - qt(1 - alpha/2, df) * se[j]
      }
      k = k + 1
    }
  }
  result <- list(low, upp)
  names(result) <- c("ciL", "ciU")
  attr(result,"conf.level") <- alpha
  if(dist == "t"){
    attr(result,"df") <- df
  }
  return(result)
}


#' @rdname cluscor_utilities
cov.dot <- function(x, y, p) {
  mux <- sum(x * apply(p, 1, sum))
  muy <- sum(y * apply(p, 2, sum))
  sigmaxy <- sum(outer(x, y, "*") * p) - mux * muy
  sigmaxy
}


#' @rdname cluscor_utilities
f <- function(x, p) {
  f <- p %*% outer(x, x, "<=")
  as.vector(f)
}


#' @rdname cluscor_utilities
f2 <- function(x, y, p, close = "xy") {
  f <- matrix(NA, nrow(p), ncol(p))
  if (close == "xy") {
    dumx <- ifelse(outer(x, x, "<="), 1, 0)
    dumy <- ifelse(outer(y, y, "<="), 1, 0)
  }
  if (close == "y") {
    dumx <- ifelse(outer(x, x, "<"), 1, 0)
    dumy <- ifelse(outer(y, y, "<="), 1, 0)
  }
  if (close == "x") {
    dumx <- ifelse(outer(x, x, "<="), 1, 0)
    dumy <- ifelse(outer(y, y, "<"), 1, 0)
  }
  if (close == "none") {
    dumx <- ifelse(outer(x, x, "<"), 1, 0)
    dumy <- ifelse(outer(y, y, "<"), 1, 0)
  }
  for (i in 1:length(x)) {
    for (j in 1:length(y)) {
      f[i, j] <- sum(outer(dumx[, i], dumy[, j], "*") * p)
    }
  }
  f
}


#' @rdname cluscor_utilities
fdot <- function(x, p) {
  f <- p %*% outer(x, x, "<=")
  fminus <- p %*% outer(x, x, "<")
  fdot <- (f + fminus)/2
  as.vector(fdot)
}


#' @rdname cluscor_utilities
fdot2 <- function(x, y, p, vars = "both") {
  fdot <- matrix(NA, length(x), length(y))
  dumx <- ifelse(outer(x, x, "<="), 1, 0)
  dumy <- ifelse(outer(y, y, "<="), 1, 0)
  dumxminus <- ifelse(outer(x, x, "<"), 1, 0)
  dumyminus <- ifelse(outer(y, y, "<"), 1, 0)
  dumx.dot <- (dumx + dumxminus)/2
  dumy.dot <- (dumy + dumyminus)/2

  for (i in 1:length(x)) {
    for (j in 1:length(y)) {
      if (vars == "both") {
        fdot[i, j] <- sum(outer(dumx.dot[, i], dumy.dot[, j], "*") * p)
      } else {
        if (vars == "x") {
          fdot[i, j] <- sum(outer(dumx.dot[, i], dumyminus[, j], "*") * p)
        } else {
          fdot[i, j] = sum(outer(dumxminus[, i], dumy.dot[, j], "*") * p)
        }
      }
    }
  }
  fdot
}


#' @rdname cluscor_utilities
var.dot <- function(z, p) {
  mu <- sum(z * p)
  sigmasq <- sum(z^2 * p) - mu^2
  sigmasq
}
