wcr_taub <- function(x, y, id, fixed = TRUE, order = 2,
                    dist = "normal", conf.level = 0.95, r = 0, ...) {

  alpha <- 1 - conf.level

  ## Subset to complete cases
  if (any(is.na(x) | is.na(y))){
    ind <- which(is.na(x) | is.na(y))
    x <- x[-ind]
    y <- y[-ind]
    id <- id[-ind]
  }
  ## Convert to numeric
  if(is.factor(x)){x <- as.numeric(x)}
  if(is.factor(y)){y <- as.numeric(y)}
  if(is.character(x)){x <- as.numeric(as.factor(x))}
  if(is.character(y)){y <- as.numeric(as.factor(y))}
  ## Convert to character
  if(!is.character(id)){id <- as.character(id)}

  {# all options
    METHOD <- "Clustered Kendall's Tau test"

    pval <- se <- numeric()

    n <- length(x)
    n.clust <- length(unique(id))

    x2 <- x[order(id)]
    y2 <- y[order(id)]
    id2 <- sort(id)

    # spearman's rho and variances based cluster-specific cdf
    dumx <- outer(x2, x2, "<=")
    x.mean <- aggrMean(dumx, id2)
    cdfx <- colMeans(x.mean)

    dumx.dot <- 0.5 * outer(x2, x2, "<=") + 0.5 * outer(x2, x2, "<")
    x.mean.dot <- aggrMean(dumx.dot, id2)
    cdfx.dot <- colMeans(x.mean.dot)

    dumy <- outer(y2, y2, "<=")
    y.mean <- aggrMean(dumy, id2)
    cdfy <- colMeans(y.mean)

    dumy.dot <- 0.5 * outer(y2, y2, "<=") + 0.5 * outer(y2, y2, "<")
    y.mean.dot <- aggrMean(dumy.dot, id2)
    cdfy.dot <- colMeans(y.mean.dot)

    xy.mean <- aggrMean(outer(x2, x2, "<=") * outer(y2, y2, "<="), id2)
    xy.mean.xminus <- aggrMean(outer(x2, x2, "<") * outer(y2, y2, "<="), id2)
    xy.mean.yminus <- aggrMean(outer(x2, x2, "<=") * outer(y2, y2, "<"), id2)
    xy.mean.minus <- aggrMean(outer(x2, x2, "<") * outer(y2, y2, "<"), id2)
    xy.mean.dot <- aggrMean((0.5 * outer(x2, x2, "<=") + 0.5 * outer(x2, x2, "<")) * (0.5 * outer(y2, y2, "<=") + 0.5 * outer(y2, y2, "<")), id2)

    cdfxy <- apply(xy.mean, 2, mean)
    cdfxy.xminus <- apply(xy.mean.xminus, 2, mean)
    cdfxy.yminus <- apply(xy.mean.yminus, 2, mean)
    cdfxy.minus <- apply(xy.mean.minus, 2, mean)
    cdfxy.dot <- apply(xy.mean.dot, 2, mean)

    x.pair <- ifelse(outer(x2, x2, ">"), 1, 0)
    x.pair <- ifelse(outer(x2, x2, "<"), -1, x.pair)
    y.pair <- ifelse(outer(y2, y2, ">"), 1, 0)
    y.pair <- ifelse(outer(y2, y2, "<"), -1, y.pair)
    xy.pair <- x.pair * y.pair

    xy.pair.vec <- gdata::lowerTriangle(aggrMean(t(aggrMean(xy.pair^2, id2)), id2), diag = FALSE)

    xy.pair[lower.tri(xy.pair, diag = T)] <- NA

    list.bdiag <- list()
    for (i in 1:n.clust) list.bdiag[[i]] <- matrix(NA, nr = table(id2)[i], nc = table(id2)[i])
    bdiag.mat <- as.matrix(Matrix::bdiag(list.bdiag))
    bdiag.mat <- ifelse(bdiag.mat == F, 0, bdiag.mat)
    xy.pair <- xy.pair + bdiag.mat
    tau.wcr <- mean(aggrMean(t(aggrMean(xy.pair, id2)), id2), na.rm = T)

    h1.0 <- (1 - 2 * cdfx.dot) * (1 - 2 * cdfy.dot)
    h1.0.bar <- tapply(h1.0, id2, mean)
    h1.a <- (1 - 2 * cdfx.dot) * (1 - 2 * cdfy.dot) + 4 * (cdfxy.dot - cdfx.dot * cdfy.dot) - tau.wcr
    h1.a.bar <- tapply(h1.a, id2, mean)
  } # all options

  if (order == 2) {
    # all order 2 estimates
    psi2 <- xy.pair
    psi2bar <- (aggrMean(t(aggrMean(psi2, id2)), id2))
    psi2bar.vec <- gdata::lowerTriangle(psi2bar, diag = FALSE)

    h1.0.xy <- matrix(h1.0.bar, n.clust, n.clust)
    h1.0.vec1 <- gdata::lowerTriangle(h1.0.xy, diag = FALSE)
    h1.0.vec2 <- gdata::lowerTriangle(t(h1.0.xy), diag = FALSE)

    h2.0 <- psi2bar.vec - h1.0.vec1 - h1.0.vec2

    h1.a.xy <- matrix(h1.a.bar, n.clust, n.clust)
    h1.a.vec1 <- gdata::lowerTriangle(h1.a.xy, diag = FALSE)
    h1.a.vec2 <- gdata::lowerTriangle(t(h1.a.xy), diag = FALSE)

    h2.a <- psi2bar.vec - tau.wcr - h1.a.vec1 - h1.a.vec2
  }  # all order 2 estimates

  {
    METHOD <- paste(METHOD, "using taub method",
                    sep = " ")
    # all taub
    taux.wcr <- mean(aggrMean(t(aggrMean(x.pair^2, id2)), id2), na.rm = T)
    tauy.wcr <- mean(aggrMean(t(aggrMean(y.pair^2, id2)), id2), na.rm = T)
    taub.wcr <- tau.wcr/sqrt(taux.wcr * tauy.wcr)

    hx <- (1 - 2 * (cdfx - cdfx.dot)) - taux.wcr
    hy <- (1 - 2 * (cdfy - cdfy.dot)) - tauy.wcr
    h1.x.bar <- tapply(hx, id2, mean)
    h1.y.bar <- tapply(hy, id2, mean)

    h1.0.taub.mat <- cbind(h1.0.bar, h1.x.bar, h1.y.bar)
    var.h1.0.taub <- t(h1.0.taub.mat) %*% h1.0.taub.mat/n.clust

    h1.a.taub.mat <- cbind(h1.a.bar, h1.x.bar, h1.y.bar)
    var.h1.a.taub <- t(h1.a.taub.mat) %*% h1.a.taub.mat/n.clust

    if (!fixed) {
      delta <- taub.wcr * c(1/tau.wcr, -1/(2 * taux.wcr), -1/(2 * tauy.wcr))
    } else {
      ratio <- taub.wcr/tau.wcr
    }

    if (order == 1) {
      METHOD <- paste(METHOD, "and a First Order Approximation of Variance",
                      sep = " ")
      var.taub.0.order1 <- 4 * var.h1.0.taub/n.clust
      var.taub.a.order1 <- 4 * var.h1.a.taub/n.clust

      if (!fixed) {
        est.var.taub1 <- delta %*% var.taub.0.order1 %*% delta
        est.var.taub2 <- delta %*% var.taub.a.order1 %*% delta

        v1 <- taub.wcr^2/est.var.taub1
        se <- sqrt(c(est.var.taub1, est.var.taub2))

        if (dist == "normal") {
          pval <- 1 - pchisq(v1, 1)
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "normal")
        }
        if (dist == "t") {
          pval <- 2 * (1 - pt(abs(taub.wcr/sqrt(est.var.taub1)), n.clust - 2))
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "t", df = n.clust - 2)
        }

      }
      if (fixed) {
        est.var.taub12 <- var.taub.0.order1[1, 1] * (ratio^2)
        est.var.taub22 <- var.taub.a.order1[1, 1] * (ratio^2)

        v1 <- taub.wcr^2/est.var.taub12
        se <- sqrt(c(est.var.taub12, est.var.taub22))

        if (dist == "normal") {
          pval <- 1 - pchisq(v1, 1)
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "normal")
        }
        if (dist == "t") {
          pval <- 2 * (1 - pt(abs(taub.wcr/sqrt(est.var.taub12)), n.clust - 2))
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "t", df = n.clust - 2)
        }

      }
    }
    if (order == 2) {
      METHOD <- paste(METHOD, "and a Second Order Approximation of Variance",
                      sep = " ")

      # all taub order 2
      xi.eq.xj.mat <- aggrMean(t(aggrMean(x.pair^2, id2)), id2)
      xbar.vec <- gdata::lowerTriangle(xi.eq.xj.mat, diag = FALSE)
      yi.eq.yj.mat <- aggrMean(t(aggrMean(y.pair^2, id2)), id2)
      ybar.vec = gdata::lowerTriangle(yi.eq.yj.mat, diag <- FALSE)

      h2.x <- xbar.vec - taux.wcr - gdata::lowerTriangle(matrix(h1.x.bar, n.clust, n.clust), diag = FALSE) - gdata::lowerTriangle(t(matrix(h1.x.bar,
                                                                                                                                           n.clust, n.clust)), diag = FALSE)
      h2.y <- ybar.vec - tauy.wcr - gdata::lowerTriangle(matrix(h1.y.bar, n.clust, n.clust), diag = FALSE) - gdata::lowerTriangle(t(matrix(h1.y.bar,
                                                                                                                                           n.clust, n.clust)), diag = FALSE)


      h2.0.taub.mat <- cbind(h2.0, h2.x, h2.y)
      h2.a.taub.mat <- cbind(h2.a, h2.x, h2.y)

      var.h2.0.taub <- t(h2.0.taub.mat) %*% h2.0.taub.mat/dim(h2.0.taub.mat)[1]
      var.h2.a.taub <- t(h2.a.taub.mat) %*% h2.a.taub.mat/dim(h2.a.taub.mat)[1]

      var.taub.0.order2 <- 4 * var.h1.0.taub/n.clust + 2 * var.h2.0.taub/(n.clust * (n.clust - 1))
      var.taub.a.order2 <- 4 * var.h1.a.taub/n.clust + 2 * var.h2.a.taub/(n.clust * (n.clust - 1))


      if (!fixed) {
        est.var.taub3 <- delta %*% var.taub.0.order2 %*% delta
        est.var.taub4 <- delta %*% var.taub.a.order2 %*% delta

        v3 <- taub.wcr^2/est.var.taub3
        se <- sqrt(c(est.var.taub3, est.var.taub4))

        if (dist == "normal") {
          pval <- 1 - pchisq(v3, 1)
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "normal")
        }
        if (dist == "t") {
          pval <- 2 * (1 - pt(abs(taub.wcr/sqrt(est.var.taub3)), n.clust - 2))
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "t", df = n.clust - 2)
        }

      }
      if (fixed) {
        est.var.taub32 <- var.taub.0.order2[1, 1] * (ratio^2)
        est.var.taub42 <- var.taub.a.order2[1, 1] * (ratio^2)

        v3 <- taub.wcr^2/est.var.taub32
        se <- sqrt(c(est.var.taub32, est.var.taub42))

        if (dist == "normal") {
          pval <- 1 - pchisq(v3, 1)
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "normal")
        }
        if (dist == "t") {
          pval <- 2 * (1 - pt(abs(taub.wcr/sqrt(est.var.taub32)), n.clust - 2))
          ci <- ci(taub.wcr, se[2], alpha = alpha, dist = "t", df = n.clust - 2)
        }

      }

    }

    if(!is.na(ci$ciL) & !is.na(ci$ciU)){
      if(ci$ciU > 1){
        warning(paste0(
          "The upper limit of the ", (1 - alpha) * 100,"% confidence interval is
          truncated at 1."
        ))
        ci$ciU <- 1
      }
      if(ci$ciL < -1){
        warning(paste0(
          "The lower limit of the ", (1 - alpha) * 100,"% confidence interval is
      truncated at -1."
        ))
        ci$ciL <- -1
      }
      if(pval < alpha & ci$ciL < 0 & ci$ciU > 0){
        warning(paste0(
          "The ", (1 - alpha) * 100,"% confidence interval is not consistent with
        statistical testing.\n"
        ))
      }
    }
    names(taub.wcr) <- names(r) <- "taub"
    results <- list(statistic = taub.wcr, p.value = pval, conf.int = ci,
                    n.clust = n.clust, nobs = n, method = METHOD,
                    dist = dist, fixed = fixed, null.value = r,
                    x = x, y = y, id = id)
    }  # tau-b

  class(results) <- "cluscor"
  return(results)
  }
