% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluscor.test.R
\name{cluscor.test}
\alias{cluscor.test}
\title{Correlation Tests for Clustered Data}
\usage{
cluscor.test(x, y, id, method = c("chisq", "rho", "tau", "taub",
  "gamma"), order = NULL, r = 0, conf.level = 0.95, fixed = NULL,
  dist = c("normal", "t"), ...)
}
\arguments{
\item{x, y}{Numeric vectors of length \code{n}. See 'details'.}

\item{id}{Character vector of length \code{n} of cluster identifications.}

\item{method}{Character. One of \code{chisq}, \code{rho}, \code{tau},
\code{taub}, or \code{gamma}}

\item{order}{Numeric. First (\code{order=1}) or second (\code{order=2}) order
expansion of the variance approximation. Default is 2 for \code{chisq},
\code{tau}, \code{taub}, and \code{gamma}. Default is 1 for \code{rho}.}

\item{r}{Numeric. Hypothesized null correlation coefficient value. Default is
0.}

\item{conf.level}{Numeric. Confidence level for the returned confidence
interval. Default is 0.95.}

\item{fixed}{Logical. Treat denominator of estimate as fixed (\code{TRUE}) or
random (\code{FALSE}). Default is \code{TRUE} for \code{rho}, \code{taub},
and \code{gamma}. It is not used for \code{chisq} or \code{tau}.}

\item{dist}{Character. Use the \code{"t"} or \code{"normal"} distribution for
calculating the p-value. \code{dist} is not used for for \code{chisq} and
the default is \code{"normal"} for the other tests.}
}
\value{
Object of class \code{cluscor} with the following components:
 \itemize{
   \item \code{p-value} - the p-value of the test.
   \item \code{estimate} - the estimate of the correlation for the chosen method.
   \item \code{conf.int} - a confidence interval around the correlation estimates.
   \item \code{method} - a character string giving the correlation or association
       chosen to be calculated.
   }
}
\description{
Tests and estimates (when appropriate) of correlations or
  associations of two variables are calculated. Each pair of variables can be
  measured on clustered data. Different cluster sizes and informative cluster
  size is allowed. Clustered versions of \enumerate{ \item Spearman’s rank
  correlation \eqn{\rho} \item Kendall’s \eqn{\tau} , \eqn{\tau_b}, and
  Goodman and Kruskal’s \eqn{\gamma} for continuous or ordinal data
  accounting for ties in the data, and \item Pearsons \eqn{\chi^2} test for
  data with two or more categories, } can be calculated. A U-statistic
  framework applied to weighted cluster resample averaging is used. The
  function allows for a first or second order expansion in the approximation
  to estimate the variance that is used in the test statistic. The test based
  on the rank statistics can be compared to a normal distribution or a
  \eqn{t} distribution. The test statistics for \eqn{\tau_b}, \eqn{\gamma},
  and \eqn{\rho} can use either the delta method to calculate the variances
  of the parameter estimates or assume the denominator of the parameter
  estimates is known and use a fixed denominator to calculate the variances.
}
\details{
Both \code{x} and \code{y} are numeric vectors of length \eqn{n}.
  For the \code{chisq} method, the values of \code{x} and \code{y} are
  categories of response and there may be 2 or more categories. For the
  \code{tau}, \code{taub}, \code{gamma}, and \code{rho} methods, \code{x} and
  \code{y} are continuous or ordinal responses. The \code{id} variable
  represents the patient or cluster identification.

  Either the first order (\code{order=1}) or second order (\code{order=2})
  expansion in the approximation to the variance can be used to calculate the
  p-value.

  When estimating the variance of \code{rho}, \code{taub}, or \code{gamma},
  the denominator can be treated as either a fixed (\code{TRUE}) or random
  quantity. There is no denominator in the estimate of the variance for
  \code{tau} therefore \code{fixed=NULL}. Additionally, either a
  \eqn{t}-distribution (\eqn{df = n.clust - 2}) or Normal distribution can be
  used for the calculation of the p-value for the tests of \code{rho},
  \code{tau}, \code{taub}, or \code{gamma}.
}
\examples{
x <- sample(1:2, 20, replace = T)
y <- sample(1:2, 20, replace = T)
id <- rep(letters[1:5], 4)
cor.test(x,y)

## Chi-sq:
(ex <- cluscor.test(x, y, id, method = "chisq"))
(ex <- cluscor.test(x, y, id, method = "chisq", order = 1))
(ex <- cluscor.test(x, y, id, method = "chisq", fixed = FALSE))

## Tau
(ex <- cluscor.test(x, y, id, method = "tau"))
(ex <- cluscor.test(x, y, id, method = "tau", dist = "t"))
(ex <- cluscor.test(x, y, id, method = "tau", order = 1))
(ex <- cluscor.test(x, y, id, method = "tau", conf.level = 0.99))
(ex <- cluscor.test(x, y, id, method = "tau", fixed = TRUE))

## Tau-b
(ex <- cluscor.test(x, y, id, method = "taub"))
(ex <- cluscor.test(x, y, id, method = "taub", dist = "t"))
(ex <- cluscor.test(x, y, id, method = "taub", order = 1))
(ex <- cluscor.test(x, y, id, method = "taub", fixed = FALSE))
(ex <- cluscor.test(x, y, id, method = "taub", conf.level = 0.9))

## Gamma
(ex <- cluscor.test(x, y, id, method = "gamma"))
(ex <- cluscor.test(x, y, id, method = "gamma", dist = "t"))
(ex <- cluscor.test(x, y, id, method = "gamma", order = 1))
(ex <- cluscor.test(x, y, id, method = "gamma", fixed = FALSE))
(ex <- cluscor.test(x, y, id, method = "gamma", conf.level = 0.975))

## Rho
(ex <- cluscor.test(x, y, id, method = "rho"))
(ex <- cluscor.test(x, y, id, method = "rho", dist = "t"))
(ex <- cluscor.test(x, y, id, method = "rho", order = 2))
(ex <- cluscor.test(x, y, id, method = "rho", fixed = FALSE))
(ex <- cluscor.test(x, y, id, method = "rho", conf.level = 0.8))
}
\references{
Shih, Joanna H., and Michael P. Fay. "Pearson's chi‐square test and rank
correlation inferences for clustered data." Biometrics 73.3 (2017): 822-834.

Our new paper

Agresti, Alan. Analysis of ordinal categorical data. Vol. 656. John Wiley &
Sons, 2010.

Hollander, Myles, and Douglas A. Wolfe. Nonparametric statistical methods.
New York, NY, USA: Wiley, 1923.
}
